/**
 * Copyright (c) 2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.admin.junit;

import java.util.Arrays;
import java.util.List;
import java.util.Properties;

import org.ow2.petals.admin.api.ContainerAdministration;
import org.ow2.petals.admin.api.artifact.Component;
import org.ow2.petals.admin.api.artifact.Logger;
import org.ow2.petals.admin.api.artifact.ServiceAssembly;
import org.ow2.petals.admin.api.artifact.SharedLibrary;
import org.ow2.petals.admin.api.exception.ContainerAdministrationException;
import org.ow2.petals.admin.api.exception.InvalidLogLevelException;
import org.ow2.petals.admin.api.exception.LoggerNotFoundException;
import org.ow2.petals.admin.api.exception.NoConnectionException;
import org.ow2.petals.admin.junit.exception.NoDomainRegisteredException;
import org.ow2.petals.admin.topology.Container;
import org.ow2.petals.admin.topology.Container.PortType;
import org.ow2.petals.admin.topology.Domain;

/**
 * <p>
 * A mock of the {@link ContainerAdministration} API.
 * </p>
 * <p>
 * This mock provides only one method requiring a connection: {@link #getSystemInfo()}.
 * </p>
 * 
 * @author "Mathieu CARROLLE - EBM WebSourcing"
 * 
 */
public class ContainerAdministrationMock implements ContainerAdministration {

    /**
     * Flag to simulate an established connection
     */
    private static volatile boolean IS_CONNECTED = false;

    private final String systemInfo;

    private final Domain domain;

    private final boolean isNullDomain;

    private final ContainerConnectionCallback connectionCallback;

    /**
     * Container on which the Petals Administration API is connected
     */
    private Container currentContainer = null;

    public ContainerAdministrationMock(final boolean isNullDomain, final Domain domain, final String systemInfo,
            final Container containerConnectedOn, final ContainerConnectionCallback connectionCallback) {
        this.domain = domain;
        this.systemInfo = systemInfo;
        this.isNullDomain = isNullDomain;
        this.currentContainer = containerConnectedOn;
        this.connectionCallback = connectionCallback;
    }

    @Override
    public void connect(final String host, final int port, final String user, final String password)
            throws ContainerAdministrationException {
        if (this.isNullDomain) {
            IS_CONNECTED = true;
        } else if (this.domain == null) {
            throw new ContainerAdministrationException(new NoDomainRegisteredException());
        } else {
            for (final Container container : this.domain.getContainers()) {
                if (container.getHost().equals(host) && container.getPorts().get(PortType.JMX).intValue() == port) {
                    IS_CONNECTED = true;
                    this.currentContainer = container;
                    this.connectionCallback.connectionSucceededOn(container);
                    return;
                }
            }
            // No container matching host:port was found in the domain
            IS_CONNECTED = false;
            throw new ContainerAdministrationException("Unknown Host");
        }
    }

    @Override
    public boolean isConnected() throws ContainerAdministrationException {
        return IS_CONNECTED;
    }

    @Override
    public void disconnect() throws NoConnectionException, ContainerAdministrationException {
        IS_CONNECTED = false;
        this.currentContainer = null;
        this.connectionCallback.connectionSucceededOn(null);
    }

    @Override
    public String getSystemInfo() throws ContainerAdministrationException {
        if (!IS_CONNECTED) {
            throw new ContainerAdministrationException("No connection establised");
        }
        return this.systemInfo;
    }

    @Override
    public void stopContainer() throws ContainerAdministrationException {
        // TODO Auto-generated method stub

    }

    @Override
    public void shutdownContainer() throws ContainerAdministrationException {
        // TODO Auto-generated method stub

    }

    @Override
    public Domain getTopology(final String containerFilterRegexp, final String securityPassPhrase,
            final boolean retrieveJbiArtifacts) throws ContainerAdministrationException {
        if (!IS_CONNECTED) {
            throw new ContainerAdministrationException("No connection establised");
        } else {
            if (this.isNullDomain == true) {
                return null;
            } else if (this.domain == null) {
                throw new ContainerAdministrationException(new NoDomainRegisteredException());
            } else {
                final boolean isOkForSensibleInfo = this.isSecurityTopologyPassphraseOk(securityPassPhrase);
                final Domain domainToReturn = new Domain(this.domain.getName());
                for (final Container container : this.domain.getContainers()) {
                    if (container.getContainerName().matches(containerFilterRegexp)) {
                        final Container containerToReturn = new Container(container.getContainerName(),
                                container.getHost(), container.getPorts(),
                                isOkForSensibleInfo ? container.getJmxUsername() : null,
                                isOkForSensibleInfo ? container.getJmxPassword() : null, container.getState());
                        domainToReturn.addContainers(Arrays.asList(new Container[] { containerToReturn }));
                        if (retrieveJbiArtifacts) {
                            for (final SharedLibrary sharedLibrary : container.getSharedLibraries()) {
                                containerToReturn.addSharedLibrary(sharedLibrary);
                            }
                            for (final Component component : container.getComponents()) {
                                containerToReturn.addComponent(component);
                            }
                            for (final ServiceAssembly serviceAssembly : container.getServiceAssemblies()) {
                                containerToReturn.addServiceAssembly(serviceAssembly);
                            }
                        }
                    }
                }
                return domainToReturn;
            }
        }
    }

    private boolean isSecurityTopologyPassphraseOk(final String securityPassPhrase)
            throws ContainerAdministrationException {

        // Check security
        final boolean isSecurityOk;
        if (securityPassPhrase == null || securityPassPhrase.isEmpty()) {
            isSecurityOk = false;
        } else {
            // TODO: The property name of the pass-phrase should be mutualized with the Petals µKernel
            final String containerTopologyPassPhrase = this.getServerProperties()
                    .getProperty("petals.topology.passphrase");
            if (containerTopologyPassPhrase == null || containerTopologyPassPhrase.isEmpty()
                    || !containerTopologyPassPhrase.equals(securityPassPhrase)) {
                isSecurityOk = false;
            } else {
                isSecurityOk = true;
            }
        }
        return isSecurityOk;
    }

    @Override
    public Properties getServerProperties() throws ContainerAdministrationException {
        if (!IS_CONNECTED) {
            throw new ContainerAdministrationException("No connection establised");
        } else {
            return this.currentContainer != null ? this.currentContainer.getLocalConfiguration() : null;
        }
    }

    @Override
    public void changeLoggerLevel(String loggerName, String loggerLevel)
            throws LoggerNotFoundException, InvalidLogLevelException, ContainerAdministrationException {
        // TODO Auto-generated method stub
    }

    @Override
    public List<Logger> getLoggers() throws ContainerAdministrationException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public void attachContainer(final String targetDomainName, final String targetContainerName,
            final String targetContainerHost, final int targetContainerPort, final String targetContainerUser,
            final String targetContainerPwd, final String securityPassPhrase) throws ContainerAdministrationException {
        // NOP
    }

    @Override
    public void detachContainer() throws ContainerAdministrationException {
        // NOP
    }

}
