/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.admin.junit;

import java.util.Arrays;

import org.junit.rules.ExternalResource;
import org.ow2.petals.admin.EndpointDirectoryAdministrationMock;
import org.ow2.petals.admin.api.ArtifactAdministration;
import org.ow2.petals.admin.api.ContainerAdministration;
import org.ow2.petals.admin.api.EndpointDirectoryAdministration;
import org.ow2.petals.admin.api.PetalsAdministration;
import org.ow2.petals.admin.api.PetalsAdministrationFactory;
import org.ow2.petals.admin.api.artifact.Artifact;
import org.ow2.petals.admin.api.artifact.Component;
import org.ow2.petals.admin.api.artifact.ServiceAssembly;
import org.ow2.petals.admin.api.artifact.SharedLibrary;
import org.ow2.petals.admin.api.artifact.lifecycle.ArtifactLifecycleFactory;
import org.ow2.petals.admin.junit.exception.MoreThanOneContainerRegisteredException;
import org.ow2.petals.admin.junit.exception.NoContainerRegisteredException;
import org.ow2.petals.admin.junit.exception.NoDomainRegisteredException;
import org.ow2.petals.admin.junit.utils.ContainerUtils;
import org.ow2.petals.admin.topology.Container;
import org.ow2.petals.admin.topology.Domain;

/**
 * The PetalsAdministrationApi rule allows creation of a mock implementation of Petals Administration API that is
 * guaranteed to be deleted when the test method finishes (whether it passes or fails):
 * 
 * <pre>
 * public static class PetalsAdministrationApi {
 *  &#064;Ruleprotected
 *  public PetalsAdministrationApi petalsAdminApi = new PetalsAdministrationApi(...);
 * 
 *  &#064;Test
 *  public void test...() throws IOException {
 *      petalsAdminApi.registerArtifact(...);
 *      ...
 *  }
 * }
 * </pre>
 */
public class PetalsAdministrationApi extends ExternalResource
        implements PetalsAdministration, ContainerConnectionCallback {

    final public static String DOMAIN_NAME = "my-domain";

    private PetalsAdministrationFactory paf = null;

    /**
     * The information system that will be returned by {@link ContainerAdministration#getSystemInfo()}
     */
    private String systemInfo = null;

    /**
     * The domain definition
     */
    private Domain domain = null;

    /**
     * A domain is registered as <code>null</code>
     */
    private boolean isNullDomain = false;

    /**
     * Container on which the Petals Administration API is connected
     */
    private Container currentContainer = null;

    @Override
    protected void before() throws Throwable {
        this.initializePetalsAdminApi();
    }

    @Override
    protected void after() {
        try {
            this.destroyPetalsAdminApi();
        } catch (final Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Initialize the mock implementation of the Petals Administration API
     */
    private void initializePetalsAdminApi() throws Exception {

        System.setProperty(PetalsAdministrationFactory.PROPERTY_NAME,
                org.ow2.petals.admin.junit.PetalsAdministrationFactory.class.getName());

        this.paf = PetalsAdministrationFactory.getInstance();
        if (this.paf instanceof org.ow2.petals.admin.junit.PetalsAdministrationFactory) {
            ((org.ow2.petals.admin.junit.PetalsAdministrationFactory) this.paf).registerAdminApi(this);
        } else {
            throw new Exception("An invalid implementation was selected as Petals Administration Factory: "
                    + paf.getClass().getName());
        }
    }

    /**
     * Free the mock implementation of the Petals Administration API
     */
    private void destroyPetalsAdminApi() throws Exception {

        this.paf = null;
        System.clearProperty(PetalsAdministrationFactory.PROPERTY_NAME);
        PetalsAdministrationFactory.close();
    }

    @Override
    public ArtifactAdministration newArtifactAdministration() {
        return new ArtifactAdministrationMock(this.domain, this.newArtifactLifecycleFactory());
    }

    @Override
    public ContainerAdministration newContainerAdministration() {
        return new ContainerAdministrationMock(this.isNullDomain, this.domain, this.systemInfo, this.currentContainer,
                this);
    }

    @Override
    public EndpointDirectoryAdministration newEndpointDirectoryAdministration() {
        return new EndpointDirectoryAdministrationMock();
    }

    @Override
    public ArtifactLifecycleFactory newArtifactLifecycleFactory() {
        return new ArtifactLifecycleFactoryMock(this.currentContainer);
    }

    /**
     * When a connection succeeded with {@link ContainerAdministration#connect(String, int, String, String)} we store
     * the container on which the Petals Administration API is connected on.
     * 
     * @param containerConnectedOn
     *            The container on which the Petals Administration API is connected on.
     */
    @Override
    public void connectionSucceededOn(final Container containerConnectedOn) {
        this.currentContainer = containerConnectedOn;
    }

    /**
     * Register a default domain.
     */
    public PetalsAdministrationApi registerDomain() {
        return this.registerDomain(new Domain(DOMAIN_NAME));
    }

    /**
     * Register the given domain.
     * 
     * @param Domain
     *            The domain to register
     */
    public PetalsAdministrationApi registerDomain(final Domain domain) {
        this.domain = domain;
        this.isNullDomain = false;

        return this;
    }

    /**
     * <p>
     * Register a default domain as <code>null</code>.
     * </p>
     * <p>
     * To use when a connection is required for a command that does not require domain information
     * </p>
     */
    public PetalsAdministrationApi registerNullDomain() {
        this.domain = null;
        this.isNullDomain = true;

        return this;
    }

    protected Domain getDomain() {
        return this.domain;
    }

    /**
     * Register a default container as defined by {@link ContainerUtils#createContainerSample()}.
     * 
     * @throws NoDomainRegisteredException
     *             No domain was previously registered, please use {@link #registerDomain(Domain)} or
     *             {@link #registerDomain()} before this method.
     */
    public PetalsAdministrationApi registerContainer() throws NoDomainRegisteredException {
        return this.registerContainer(ContainerUtils.createContainerSample());
    }

    /**
     * Register a container in the current domain.
     * 
     * @param Container
     *            The container to register
     * @throws NoDomainRegisteredException
     *             No domain was previously registered, please use {@link #registerDomain(Domain)} or
     *             {@link #registerDomain()} before this method.
     */
    public PetalsAdministrationApi registerContainer(final Container container) throws NoDomainRegisteredException {
        if (this.domain == null) {
            throw new NoDomainRegisteredException();
        } else {
            this.domain.addContainers(Arrays.asList(new Container[] { container }));
        }

        return this;
    }

    protected Container getCurrentContainer() {
        return this.currentContainer;
    }

    /**
     * Register an artifact in the current domain managed with the Petals Administration API Mock.
     * 
     * @param artifact
     *            The artifact to register
     * @throws NoDomainRegisteredException
     *             No domain was previously registered, please use {@link #registerDomain(Domain)} or
     *             {@link #registerDomain()} before this method.
     * @throws NoContainerRegisteredException
     *             No container was previously registered, please use {@link #registerContainer(Container)} or
     *             {@link #registerContainer()} before this method.
     * @throws MoreThanOneContainerRegisteredException
     *             Several containers are registered in the current domain, please use
     *             {@link #registerArtifact(Artifact, Container) to register your artifact for the given container.}
     */
    public PetalsAdministrationApi registerArtifact(final Artifact artifact) throws NoContainerRegisteredException,
            NoDomainRegisteredException, MoreThanOneContainerRegisteredException {

        if (this.domain == null) {
            throw new NoDomainRegisteredException();
        } else if (this.domain.getContainers() == null || this.domain.getContainers().size() == 0) {
            throw new NoContainerRegisteredException();
        } else if (this.domain.getContainers().size() > 1) {
            throw new MoreThanOneContainerRegisteredException();
        } else {
            return this.registerArtifact(artifact, this.domain.getContainers().get(0));
        }
    }

    /**
     * Register an artifact in the given container of the current domain managed with the Petals Administration API
     * Mock.
     * 
     * @param artifact
     *            The artifact to register
     * @param container
     *            The container where the artifact must be registered
     */
    public PetalsAdministrationApi registerArtifact(final Artifact artifact, final Container container) {

        if (artifact instanceof SharedLibrary) {
            container.addSharedLibrary((SharedLibrary) artifact);
        } else if (artifact instanceof Component) {
            container.addComponent((Component) artifact);
        } else if (artifact instanceof ServiceAssembly) {
            container.addServiceAssembly((ServiceAssembly) artifact);
        }

        return this;
    }

    /**
     * Register a value returned by {@link ContainerAdministration#getSystemInfo()}
     * 
     * @param systemInfo
     *            The information system that will be returned by {@link ContainerAdministration#getSystemInfo()}
     */
    public void registerSystemInfo(final String systemInfo) {
        this.systemInfo = systemInfo;
    }

    protected String getSystemInfo() {
        return this.systemInfo;
    }

}