/**
 * Copyright (c) 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.admin.junit;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;

import org.junit.FixMethodOrder;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runners.MethodSorters;
import org.ow2.petals.admin.api.artifact.ArtifactState.State;
import org.ow2.petals.admin.api.artifact.Component;
import org.ow2.petals.admin.api.artifact.Component.ComponentType;
import org.ow2.petals.admin.api.artifact.SharedLibrary;
import org.ow2.petals.admin.junit.exception.MoreThanOneContainerRegisteredException;
import org.ow2.petals.admin.junit.exception.NoContainerRegisteredException;
import org.ow2.petals.admin.junit.exception.NoDomainRegisteredException;

/**
 * Unit tests of {@link PetalsAdministrationApi}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
// MethodSorter 'NAME_ASCENDING' required to check that the artifact registry is correctly cleared at the end of a unit
// test.
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class PetalsAdministrationApiTest {

    /**
     * A Petals Administration Mock dedicated to the test 'adminApiRuleClearedAtTheEnd' to check that the
     * PetalsAdministrationApi JUnit rule is correctly cleared when leaving the unit test.
     */
    @Rule
    public final PetalsAdministrationApi globalAdminApi = new PetalsAdministrationApi();

    @Rule
    public final PetalsAdministrationApi adminApiOne = new PetalsAdministrationApi();

    @Rule
    public final PetalsAdministrationApi adminApiTwo = new PetalsAdministrationApi();

    /**
     * <p>
     * Check that the PetalsAdministrationApi JUnit rule is correctly cleared when leaving the unit test.
     * </p>
     * <p>
     * In the test {@link #adminApiRuleClearedAtTheEnd_0()}, an artifact is added to the registry, the check will be
     * done in {@link #adminApiRuleClearedAtTheEnd_1()}.
     * </p>
     */
    @Test
    public void adminApiRuleClearedAtTheEnd_0() throws NoContainerRegisteredException, NoDomainRegisteredException,
            MoreThanOneContainerRegisteredException {

        final Component component = new Component("component-name", ComponentType.BC);
        component.setState(State.STARTED);
        this.globalAdminApi.registerDomain().registerContainer().registerArtifact(component);

        // The check is done in adminApiRuleClearedAtTheEnd_1
    }

    /**
     * See {@link #adminApiRuleClearedAtTheEnd_0()}
     */
    @Test
    public void adminApiRuleClearedAtTheEnd_1() throws NoContainerRegisteredException, NoDomainRegisteredException,
            MoreThanOneContainerRegisteredException {

        assertEquals("System info not reset", null, this.globalAdminApi.getSystemInfo());
        assertEquals("Domain not reset", null, this.globalAdminApi.getDomain());
        assertEquals("Current container not reset", null, this.globalAdminApi.getCurrentContainer());

        // The registration of the same component should succeeds
        final Component componentOne = new Component("component-name", ComponentType.BC);
        componentOne.setState(State.STARTED);
        this.globalAdminApi.registerDomain().registerContainer().registerArtifact(componentOne);

    }

    /**
     * Check that the isolation of two admin Api Rule.
     */
    @Test
    public void isolation() throws NoContainerRegisteredException,
            NoDomainRegisteredException, MoreThanOneContainerRegisteredException {

        final Component componentOne = new Component("component-name", ComponentType.BC);
        componentOne.setState(State.STARTED);
        this.adminApiOne.registerDomain().registerContainer().registerArtifact(componentOne);

        assertEquals(1, this.adminApiOne.getDomain().getContainers().get(0).getComponents().size());
        assertNull(this.adminApiTwo.getDomain());

        this.adminApiTwo.registerDomain().registerContainer()
                .registerArtifact(new SharedLibrary("shared-library", "version"));

        assertEquals(1, this.adminApiOne.getDomain().getContainers().get(0).getComponents().size());
        assertEquals(0, this.adminApiOne.getDomain().getContainers().get(0).getSharedLibraries().size());
        assertEquals(1, this.adminApiTwo.getDomain().getContainers().get(0).getSharedLibraries().size());
        assertEquals(0, this.adminApiTwo.getDomain().getContainers().get(0).getComponents().size());
    }

}
